@@ -16,7 +16,7 @@ import pprint
 from .utils import newoverwrite, newcopyover, ensuredirs, indent, indentstr, \
     RunControl, NotSpecified
 from .plugins import Plugin
-from . import typesystem as ts
+from .typesystem import TypeSystem
 
 if sys.version_info[0] >= 3: 
     basestring = str
@@ -171,7 +171,7 @@ class Set{clsname}(_Set{clsname}, collections.Set):
         return "set([" + ", ".join([repr(i) for i in self]) + "])"
 
 '''
-def genpyx_set(t):
+def genpyx_set(t, ts):
     """Returns the pyx snippet for a set of type t."""
     t = ts.canon(t)
     kw = dict(clsname=ts.cython_classname(t)[1], humname=ts.humanname(t)[1], 
@@ -199,7 +199,7 @@ cdef class _Set{clsname}:
 
 
 """
-def genpxd_set(t):
+def genpxd_set(t, ts):
     """Returns the pxd snippet for a set of type t."""
     return _pxdset.format(clsname=ts.cython_classname(t)[1], ctype=ts.cython_ctype(t))
 
@@ -216,13 +216,13 @@ def test_set_{fncname}():
     assert_true({3} not in s)
 
 """
-def gentest_set(t):
+def gentest_set(t, ts):
     """Returns the test snippet for a set of type t."""
     t = ts.canon(t)
     return _testset.format(*[repr(i) for i in testvals[t]], 
                            clsname=ts.cython_classname(t)[1],
                            fncname=ts.cython_functionname(t)[1],
-                           stlcontainers=ts.STLCONTAINERS)
+                           stlcontainers=ts.stlcontainers)
 
 #
 # Maps
@@ -336,6 +336,8 @@ cdef class _Map{tclsname}{uclsname}:
 {tpy2cbody.indent8}
 {upy2cbody.indent8}
         item = pair[{tctype}, {uctype}]({tpy2crtn}, {upy2crtn})
+        if 0 < self.map_ptr.count({tpy2crtn}):
+            self.map_ptr.erase({tpy2crtn})
         self.map_ptr.insert(item)
 
     def __delitem__(self, key):
@@ -368,7 +370,7 @@ class Map{tclsname}{uclsname}(_Map{tclsname}{uclsname}, collections.MutableMappi
         return "{{" + ", ".join(["{{0}}: {{1}}".format(repr(key), repr(value)) for key, value in self.items()]) + "}}"
 
 '''
-def genpyx_map(t, u):
+def genpyx_map(t, u, ts):
     """Returns the pyx snippet for a map of type <t, u>."""
     t = ts.canon(t)
     u = ts.canon(u)
@@ -407,7 +409,7 @@ cdef class _Map{tclsname}{uclsname}:
 
 
 """
-def genpxd_map(t, u):
+def genpxd_map(t, u, ts):
     """Returns the pxd snippet for a set of type t."""
     t = ts.canon(t)
     u = ts.canon(u)
@@ -471,7 +473,7 @@ def test_map_{tfncname}_{ufncname}():
         assert{array}_equal(m[{1}], {5})
 
 """
-def gentest_map(t, u):
+def gentest_map(t, u, ts):
     """Returns the test snippet for a map of type t."""
     t = ts.canon(t)
     u = ts.canon(u)
@@ -490,7 +492,7 @@ def gentest_map(t, u):
                            uclsname=ts.cython_classname(u)[1],
                            tfncname=ts.cython_functionname(t)[1], 
                            ufncname=ts.cython_functionname(u)[1], 
-                           array=a, stlcontainers=ts.STLCONTAINERS)
+                           array=a, stlcontainers=ts.stlcontainers)
 
 
 #
@@ -682,7 +684,7 @@ cdef long pyxd_{fncname}_type_hash(object self):
     return id(self)
 
 cdef PyMemberDef pyxd_{fncname}_type_members[1]
-pyxd_{fncname}_type_members[0] = PyMemberDef(NULL)
+pyxd_{fncname}_type_members[0] = PyMemberDef(NULL, 0, 0, 0, NULL)
 
 cdef PyGetSetDef pyxd_{fncname}_type_getset[1]
 pyxd_{fncname}_type_getset[0] = PyGetSetDef(NULL)
@@ -724,7 +726,8 @@ c_xd_{fncname}_descr.type_num = 0    # type_num, assigned at registration
 c_xd_{fncname}_descr.elsize = sizeof({ctype})  # elsize, 
 c_xd_{fncname}_descr.alignment = 8  # alignment
 c_xd_{fncname}_descr.subarray = NULL  # subarray
-c_xd_{fncname}_descr.fields = <PyObject *> None  # fields
+c_xd_{fncname}_descr.fields = NULL  # fields
+c_xd_{fncname}_descr.names = NULL
 (<PyArray_Descr *> c_xd_{fncname}_descr).f = <PyArray_ArrFuncs *> &PyXD_{clsname}_ArrFuncs  # f == PyArray_ArrFuncs
 
 cdef object xd_{fncname}_descr = <object> (<void *> c_xd_{fncname}_descr)
@@ -737,14 +740,14 @@ dtypes[xd_{fncname}_num] = xd_{fncname}
 
 """
 
-def genpyx_vector(t):
+def genpyx_vector(t, ts):
     """Returns the pyx snippet for a vector of type t."""
     t = ts.canon(t)
     kw = dict(clsname=ts.cython_classname(t)[1], humname=ts.humanname(t)[1], 
               fncname=ts.cython_functionname(t)[1], 
               ctype=ts.cython_ctype(t), pytype=ts.cython_pytype(t), 
-              cytype=ts.cython_cytype(t), stlcontainers=ts.STLCONTAINERS, 
-              extra_types=ts.EXTRA_TYPES)
+              cytype=ts.cython_cytype(t), stlcontainers=ts.stlcontainers, 
+              extra_types=ts.extra_types)
     t0 = t
     while not isinstance(t0, basestring):
         t0 = t[0]
@@ -775,7 +778,7 @@ cdef void pyxd_{fncname}_copyswap(void * dest, void * src, int swap, void * arr)
 cdef np.npy_bool pyxd_{fncname}_nonzero(void * data, void * arr)
 """
 
-def genpxd_vector(t):
+def genpxd_vector(t, ts):
     """Returns the pxd snippet for a vector of type t."""
     t = ts.canon(t)
     kw = dict(clsname=ts.cython_classname(t)[1], humname=ts.humanname(t)[1], 
@@ -803,14 +806,14 @@ def test_vector_{fncname}():
     print(a)
 
 """
-def gentest_vector(t):
+def gentest_vector(t, ts):
     """Returns the test snippet for a set of type t."""
     t = ts.canon(t)
     if ('vector', t, 0) in testvals:
         s = _testvector.format(*[repr(i) for i in testvals['vector', t, 0]], 
                                clsname=ts.cython_classname(t)[1],
                                fncname=ts.cython_functionname(t)[1],
-                               stlcontainers=ts.STLCONTAINERS)
+                               stlcontainers=ts.stlcontainers)
     else:
         s = ""
     return s
@@ -852,6 +855,12 @@ np.import_array()
 
 cimport {extra_types}
 
+# Cython Imports For Types
+{cimports}
+
+# Imports For Types
+{imports}
+
 dtypes = {{}}
 
 if PY_MAJOR_VERSION >= 3:
@@ -867,15 +876,25 @@ cdef extern from *:
     cdef void emit_endif "#endif //" ()
 
 """
-def genpyx(template, header=None):
+def genpyx(template, header=None, ts=None):
+    ts = ts or TypeSystem()
     """Returns a string of a pyx file representing the given template."""
     pyxfuncs = dict([(k[7:], v) for k, v in globals().items() \
                     if k.startswith('genpyx_') and callable(v)])
     pyx = _pyxheader if header is None else header
-    pyx = pyx.format(extra_types=ts.EXTRA_TYPES)
     with ts.swap_stlcontainers(None):
+        import_tups = set()
+        cimport_tups = set()
         for t in template:
-            pyx += pyxfuncs[t[0]](*t[1:]) + "\n\n" 
+            for arg in t[1:]:
+                ts.cython_import_tuples(arg, import_tups)
+                ts.cython_cimport_tuples(arg, cimport_tups)
+        imports = "\n".join(ts.cython_import_lines(import_tups))
+        cimports = "\n".join(ts.cython_cimport_lines(cimport_tups))
+        pyx = pyx.format(extra_types=ts.extra_types, cimports=cimports, 
+                         imports=imports)
+        for t in template:
+            pyx += pyxfuncs[t[0]](*t[1:], ts=ts) + "\n\n" 
     return pyx
 
 
@@ -908,6 +927,10 @@ cimport {extra_types}
 
 cimport numpy as np
 
+
+# Cython Imports For Types
+{cimports}
+
 cdef extern from "Python.h":
     ctypedef Py_ssize_t Py_ssize_t
 
@@ -916,9 +939,6 @@ cdef extern from "Python.h":
     cdef long Py_TPFLAGS_CHECKTYPES
     cdef long Py_TPFLAGS_HEAPTYPE
 
-    ctypedef struct PyMemberDef:
-        char * name
-
     ctypedef struct PyGetSetDef:
         char * name
 
@@ -943,6 +963,15 @@ cdef extern from "Python.h":
         int (*tp_compare)(object, object)      # Python 2
         void * (*tp_reserved)(object, object)  # Python 3
 
+# structmember.h isn't included in Python.h for some reason
+cdef extern from "structmember.h":
+    ctypedef struct PyMemberDef:
+        char * name
+        int type
+        Py_ssize_t offset
+        int flags
+        char * doc
+
 cdef extern from "numpy/arrayobject.h":
 
     ctypedef object (*PyArray_GetItemFunc)(void *, void *)
@@ -1006,6 +1035,7 @@ cdef extern from "numpy/arrayobject.h":
         int alignment
         PyArray_ArrayDescr * subarray
         PyObject * fields
+        PyObject * names
         PyArray_ArrFuncs * f
 
     cdef int PyArray_RegisterDataType(PyArray_Descr *)
@@ -1020,14 +1050,21 @@ cdef extern from "{extra_types}.h" namespace "{extra_types}":
         void deall(T *) nogil except +
 
 """
-def genpxd(template, header=None):
+def genpxd(template, header=None, ts=None):
     """Returns a string of a pxd file representing the given template."""
+    ts = ts or TypeSystem()
     pxdfuncs = dict([(k[7:], v) for k, v in globals().items() \
                     if k.startswith('genpxd_') and callable(v)])
     pxd = _pxdheader if header is None else header
-    pxd = pxd.format(extra_types=ts.EXTRA_TYPES)
+    with ts.swap_stlcontainers(None):
+        cimport_tups = set()
+        for t in template:
+            for arg in t[1:]:
+                ts.cython_cimport_tuples(arg, cimport_tups, set(['c']))
+        cimports = "\n".join(ts.cython_cimport_lines(cimport_tups))
+        pxd = pxd.format(extra_types=ts.extra_types, cimports=cimports)
     for t in template:
-        pxd += pxdfuncs[t[0]](*t[1:]) + "\n\n" 
+        pxd += pxdfuncs[t[0]](*t[1:], ts=ts) + "\n\n" 
     return pxd
 
 
@@ -1058,22 +1095,24 @@ _testfooter = '''if __name__ == '__main__':
     nose.run()
 '''
 
-def gentest(template, header=None, package='..'):
+def gentest(template, header=None, package='..', ts=None):
     """Returns a string of a test file representing the given template."""
+    ts = ts or TypeSystem()
     testfuncs = dict([(k[8:], v) for k, v in globals().items() \
                     if k.startswith('gentest_') and callable(v)])
     test = _testheader if header is None else header
-    test = test.format(stlcontainers=ts.STLCONTAINERS, package=package)
+    test = test.format(stlcontainers=ts.stlcontainers, package=package)
     for t in template:
-        test += testfuncs[t[0]](*t[1:]) + "\n\n" 
+        test += testfuncs[t[0]](*t[1:], ts=ts) + "\n\n" 
     test += _testfooter
     return test
 
 
 def genfiles(template, fname='temp', pxdname=None, testname=None, 
              pyxheader=None, pxdheader=None, testheader=None, package='..', 
-             verbose=False):
+             ts=None, verbose=False):
     """Generates all cython source files needed to create the wrapper."""
+    ts = ts or TypeSystem()
     # munge some filenames
     fname = fname[:-4] if fname.endswith('.pyx') else fname
     pxdname = fname if pxdname is None else pxdname
@@ -1087,9 +1126,9 @@ def genfiles(template, fname='temp', pxdname=None, testname=None,
         if t[0] == 'vector':
             ts.register_numpy_dtype(t[1])
 
-    pyx = genpyx(template, pyxheader)
-    pxd = genpxd(template, pxdheader)
-    test = gentest(template, testheader, package)
+    pyx = genpyx(template, pyxheader, ts=ts)
+    pxd = genpxd(template, pxdheader, ts=ts)
+    test = gentest(template, testheader, package, ts=ts)
 
     newoverwrite(pyx, fname, verbose)
     newoverwrite(pxd, pxdname, verbose)
@@ -1111,16 +1150,26 @@ class XDressPlugin(Plugin):
         make_stlcontainers=True,
         )
 
+    rcdocs = {
+        "stlcontainers": "List of C++ standard library containers to wrap.",
+        "stlcontainers_module": ("Module name for C++ standard library "
+                                 "container wrappers."),
+        "make_stlcontainers": ("Flag for enabling / disabling creating the "
+                               "C++ standard library container wrappers."),
+        }
+
     def update_argparser(self, parser):
         parser.add_argument('--stlcontainers-module', action='store', 
-                        dest='stlcontainers_module', help="stlcontainers module name")
+                dest='stlcontainers_module', help=self.rcdocs["stlcontainers_module"])
         parser.add_argument('--make-stlcontainers', action='store_true',
                     dest='make_stlcontainers', help="make C++ STL container wrappers")
         parser.add_argument('--no-make-stlcontainers', action='store_false',
               dest='make_stlcontainers', help="don't make C++ STL container wrappers")
 
     def setup(self, rc):
-        ts.STLCONTAINERS = rc.stlcontainers_module
+        print("stlwrap: registering C++ standard library types")
+        ts = rc.ts
+        ts.stlcontainers = rc.stlcontainers_module
         # register dtypes
         for t in rc.stlcontainers:
             if t[0] == 'vector':
@@ -1137,7 +1186,7 @@ class XDressPlugin(Plugin):
         testname = os.path.join(rc.packagedir, 'tests', testname)
         ensuredirs(testname)
         genfiles(rc.stlcontainers, fname=fname, testname=testname, package=rc.package, 
-                 verbose=rc.verbose)
+                 ts=rc.ts, verbose=rc.verbose)
 
 
 #
